#!/bin/bash

service iptables status && service iptables stop
setenforce 0
service openais restart


sname=$(basename $0)

ulimit -c unlimited

QPID_DEPS=`find /usr/share/java/qpid-deps/*.jar | tr '\n' ":"`
#QPID_JARS=`find /usr/share/java -name 'qpid-client*.jar' -or -name 'qpid-common*.jar' | tr '\n' ":"`
QPID_JARS="/root/qpid-client-0.9.jar:/root/qpid-common-0.9.jar:"
LOG4J="/usr/share/java/log4j.jar"
QPID_CLASSPATH=$QPID_DEPS$LOG4J:$QPID_JARS
NR_LOOPS=10
[[ "$1" =~ "^[0-9]+$" ]] && NR_LOOPS=$1

QPORT2=10000
QNAME="my-queUE"



function lognl ()
{
  echo "[$(date +%H:%M:%S)] $*"
}


function compile ()
{
  javac -cp "$QPID_DEPS$LOG4J:$QPID_JARS" TxSender.java
  #java -cp "$QPID_DEPS$LOG4J:$QPID_JARS" TxSender
}

function drain_queue()
{
  java -cp "$QPID_DEPS$LOG4J:$QPID_JARS" DrainQueue $1
}

function run_the_batch ()
{
  
  # clean stores
  rm -rf ./dd*
  
  # node 1
  local cparams="-t --auth=no --truncate yes --cluster-name=bz667428_at_$(hostname) --log-enable=info+"
  qpidd ${cparams} -p 5672 --data-dir=./dd5672 &> ./qpidd.5672.$1.log &
  QPIDD_N1=$!
  
  while true; do
    [ "$(netstat -nlp | grep -c qpidd)" == "1" ] & break
  done
  
  sleep 0.1
  qpid-cluster | awk '{print "  " $0}'
  
  # node 2
  qpidd ${cparams} -p ${QPORT2} --data-dir=./dd${QPORT2}  &> ./qpidd2.${QPORT2}.$1.log &
  QPIDD_N2=$!
  
  while true; do
    [ "$(netstat -nlp | grep -c qpidd)" == "2" ] & break
  done

  sleep 0.1
  lognl "Brokers up:"
  qpid-cluster | awk '{print "  " $0}'
  netstat -nlp | grep qpidd | awk '{print "  " $0}'
  
  sleep 4  

  #lognl "Clean-up the queue"
  #qpid-config del queue ${QNAME} --force-if-not-empty

  lognl "Check whether the queue is empty:"
  ./qc_list_objects.py -p ${QPORT2} -qv | grep "${QNAME}" | awk '{print "  " $0}'

  # run the java client
  lognl  "Running tha client:"
  java -cp "$QPID_DEPS$LOG4J:$QPID_JARS" TxSender &> cli.$1.log &
  
  
  # random wait
  lognl  "Preparing the moment for the failover:"
  sts=${SECONDS}
  dur=$((4+(${RANDOM}%2)))
  while true; do
    ./qc_list_objects.py -p ${QPORT2} -qv | grep "${QNAME}" | awk '{print "  " $0}'
    [ "$(( ${SECONDS} - ${sts} ))" -gt "${dur}" ] && break
  done
  sync
  lognl "$(grep -c 'message:' cli.$1.log) messages sent by client) ... going to kill"
  
  # kill the first broker
  lognl "Broker 5672 going to be killed..."
  kill -2 $QPIDD_N1
  lognl ".killed"

  while true; do
    [ "$(netstat -nlp | grep -c qpidd)" == "1" ] & break
    ./qc_list_objects.py -p ${QPORT2} -qv | grep "${QNAME}" | awk '{print "  " $0}'
  done
  lognl ".process gone"

  netstat -nlp | grep qpidd | awk '{print "  " $0}'
  
  ./qc_list_objects.py -p ${QPORT2} -qv | grep "${QNAME}"
  
  lognl "Waiting for client:"
  while true; do
    ./qc_list_objects.py -p ${QPORT2} -qv | grep "${QNAME}" | awk '{print "  " $0}'
    ps $! &>/dev/null || break
    sleep 0.2
  done
  
  ./qc_list_objects.py -p ${QPORT2} -qv | grep "${QNAME}" | awk '{print "  " $0}'


  lognl "Read-out the queue:"
  #/usr/share/doc/python-qpid-0.7.946106/examples/api/drain -b localhost:${QPORT2} "${QNAME}; {delete:always}"
  drain_queue ${QPORT2}

  lognl "End broker check I:"
  service openais status
  netstat -nlp | grep -E '(qpidd)' | awk '{print "  " $0}'
  
  if [ "$2" == "yes" ]; then
    
    lognl "Kill all broker instances:"
    killall qpidd

    sleep 2
  fi
  

  lognl "End broker check II:"
  service openais status
  netstat -nlp | grep -E '(qpidd)' | awk '{print "  " $0}'
}



# main

{


killall -9 qpidd
rm -f *.log
compile


for ((i=0;i<${NR_LOOPS};i++)); do
  lognl "run(${i}) ----------------------------------------------------------begin-"
  i_kar="yes"
  #[ "${i}" == "$((${NR_LOOPS} - 1))" ] && i_kar=no
  run_the_batch ${i} ${i_kar}
  lognl "run(${i}) ------------------------------------------------------------end-"
done




#setenforce 1

} 2>&1 | tee ${sname}.log


# eof
